<?php

namespace Drupal\menu_manager\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\system\Entity\Menu;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for adding links to a menu.
 */
class AddLinksToMenuForm extends FormBase {

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new AddLinksToMenuForm.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(RouteMatchInterface $route_match, MessengerInterface $messenger) {
    $this->routeMatch = $route_match;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'menu_manager_add_links_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $menu = $this->routeMatch->getParameter('menu');

    if (!$menu instanceof Menu) {
      $this->messenger->addError($this->t('Menu not found.'));
      return [];
    }

    $form['menu_links'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Menu links'),
      '#description' => $this->t('Add one menu item per line. Format: <em>Label | Path</em>. Use spaces for hierarchy, e.g., "  Subitem | /subpath".'),
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add links'),
    ];

    $form_state->set('menu', $menu);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $menu = $form_state->get('menu');
    $menu_links_text = $form_state->getValue('menu_links');

    $lines = explode("\n", $menu_links_text);
    $parent_stack = [];
    $weight_counter = 0;

    foreach ($lines as $line) {
      $line = rtrim($line);
      if (empty($line)) {
        continue;
      }

      $depth = strlen($line) - strlen(ltrim($line));
      $line = ltrim($line);

      [$label, $path] = array_map('trim', explode('|', $line) + [1 => '']);

      if (empty($label)) {
        continue;
      }

      switch ($path) {
        case '<front>':
          $path = 'internal:/';
          break;

        case '<nolink>':
          $path = 'route:<nolink>';
          break;

        case '<none>':
          $path = 'route:<none>';
          break;

        case '<button>':
          $path = 'route:<button>';
          break;

        default:
          if (str_starts_with($path, '/')) {
            $path = 'internal:' . $path;
          }
          elseif (!str_starts_with($path, 'http') && !str_starts_with($path, 'route:')) {
            $this->messenger()->addError($this->t('Invalid path "@path". Menu link "@label" was not added.', [
              '@path' => $path,
              '@label' => $label,
            ]));
            break;
          }
          break;
      }

      try {
        $menu_link = MenuLinkContent::create([
          'title' => $label,
          'link' => ['uri' => $path],
          'menu_name' => $menu->id(),
          'parent' => $parent_stack[$depth - 1] ?? NULL,
          'weight' => $weight_counter,
        ]);
        $menu_link->save();

        $parent_stack[$depth] = $menu_link->getPluginId();
        $weight_counter++;
      }
      catch (\Exception $e) {
        $this->messenger()->addError($this->t('Failed to add menu link "@label": @message', [
          '@label' => $label,
          '@message' => $e->getMessage(),
        ]));
      }
    }

    $this->messenger()->addMessage($this->t('Menu links have been added successfully.'));
  }

  /**
   * Converts user input paths to internal URIs.
   *
   * @param string $path
   *   The path input by the user.
   *
   * @return string
   *   The converted internal URI.
   */
  protected function convertToInternalUri($path) {
    if (strpos($path, '/') === 0) {
      return 'internal:' . $path;
    }
    return Url::fromUserInput($path)->toUriString();
  }

}
